@@ -1,6 +1,6 @@
 # $Author: ddumont $
-# $Date: 2009-03-12 13:33:03 +0100 (Thu, 12 Mar 2009) $
-# $Revision: 893 $
+# $Date: 2009-09-04 17:59:46 +0200 (Fri, 04 Sep 2009) $
+# $Revision: 1015 $
 
 #    Copyright (c) 2008 Dominique Dumont.
 #
@@ -32,8 +32,9 @@ use vars qw/$VERSION $icon_path/ ;
 use subs qw/menu_struct/ ;
 use Tk::Dialog ;
 use Tk::Photo ;
+use Tk::Balloon ;
 
-$VERSION = sprintf "1.%04d", q$Revision: 893 $ =~ /(\d+)/;
+$VERSION = sprintf "1.%04d", q$Revision: 1015 $ =~ /(\d+)/;
 
 Construct Tk::Widget 'ConfigModelHashEditor';
 
@@ -42,7 +43,7 @@ my @fxe1 = qw/-fill x    -expand 1/ ;
 my @fx   = qw/-fill x    / ;
 my $logger = Log::Log4perl::get_logger(__PACKAGE__);
 
-my $entry_width = 20 ;
+my $entry_width = 15 ;
 
 my $up_img;
 my $down_img;
@@ -62,6 +63,7 @@ sub Populate {
     my $hash = $cw->{hash} = delete $args->{-item} 
       || die "HashEditor: no -item, got ",keys %$args;
     delete $args->{-path} ;
+    $cw->{store_cb} = delete $args->{-store_cb} || die __PACKAGE__,"no -store_cb" ;
 
     unless (defined $up_img) {
 	$up_img   = $cw->Photo(-file => $icon_path.'up.png');
@@ -76,7 +78,8 @@ sub Populate {
 
     my $elt_frame = $elt_button_frame->Frame(qw/-relief raised -borderwidth 2/)
                                      ->pack(@fbe1,-side => 'left') ;
-    $elt_frame -> Label(-text => $hash->element_name.' elements') -> pack() ;
+    $elt_frame -> Label(-text => $hash->element_name.' elements')
+      -> pack(@fx) ;
 
     my $tklist = $elt_frame ->Scrolled ( 'Listbox',
 					 -selectmode => 'single',
@@ -87,44 +90,65 @@ sub Populate {
 
     $tklist->insert( end => $hash->get_all_indexes) ;
 
-    my $right_frame = $elt_button_frame->Frame->pack(@fxe1, -side => 'left');
+    my $right_frame = $elt_button_frame->Frame
+      ->pack(@fxe1, qw/-side right -anchor n/);
 
-    $cw->add_info($cw) ;
-    $cw->add_help_frame() ;
-    $cw->add_help(class   => $hash->parent->get_help) ;
-    $cw->add_help(element => $hash->parent->get_help($hash->element_name)) ;
+    $cw->add_info() ;
+    $cw->add_summary_and_description($hash) ;
 
-    my $add_item = '';
-    my $add_frame = $right_frame->Frame->pack( @fxe1);
+    my $item_frame = $right_frame->Frame(qw/-borderwidth 1 -relief groove/)
+      ->pack( @fxe1);
 
-    my $add_str = $hash->ordered ? "after selection " : '' ;
-    $add_frame -> Button(-text => "Add item $add_str:",
-			 -command => sub {$cw->add_entry($add_item);},
-			 -anchor => 'e',
-			)->pack(-side => 'left', @fx);
-    $add_frame -> Entry (-textvariable => \$add_item, -width => $entry_width)
-               -> pack  (-side => 'left') ;
+    my $balloon = $cw->Balloon(-state => 'balloon') ;
 
-    my $cp_frame = $right_frame->Frame->pack( @fxe1);
-    my $cp_item = '';
-    $cp_frame -> Button(-text => 'Copy selected item into:',
-			-command => sub {$cw->copy_selected_in($cp_item);},
-			-anchor => 'e',
-		       )
-              -> pack(-side => 'left', @fx);
-    $cp_frame -> Entry (-textvariable => \$cp_item, -width => $entry_width)
-              -> pack  (-side => 'left') ;
+    my $item = '';
+    my $keep = 0 ;
+    my $label_frame = $item_frame->Frame->pack( @fxe1, qw/-side top -anchor n/);
+    $label_frame -> Label (-text => 'Item:')->pack(@fxe1,-side => 'left') ;
+    my $keep_b = $label_frame -> Checkbutton (-variable => \$keep, 
+					    -text => 'keep')
+      -> pack  (-side => 'left') ;
+    $balloon->attach($keep_b, 
+		     -msg => 'keep entry in widget after add, move or copy');
+
+    my $entry = $item_frame -> Entry (-textvariable => \$item )
+      -> pack  (@fxe1, qw/-side top -anchor n/) ;
+    $balloon -> attach($entry, 
+		       -msg => 'enter item name to add, copy to, or move to') ;
+
+    my $button_frame = $item_frame->Frame->pack( qw/-side top -anchor n/ );
+
+    my $addb = $button_frame 
+      -> Button(-text => "Add",
+		-command => sub { $cw->add_entry($item); 
+				  $item = '' unless $keep;
+			      },
+		-anchor => 'e',
+	       )->pack(qw/-side left/);
+    my $add_str = $hash->ordered ? " after selection" : '' ;
+    $balloon->attach($addb,
+		     -msg => "add entry".$add_str);
+
+    my $cp_b = $button_frame 
+      -> Button(-text => 'Copy',
+		-command => sub { $cw->copy_selected_in($item); 
+				  $item = '' unless $keep;},
+		-anchor => 'e',
+	       )
+	-> pack(qw/-side right/);
+    $balloon->attach($cp_b,
+		     -msg => "copy selected item in entry");
 
 
-    my $mv_frame = $right_frame->Frame->pack( @fxe1);
-    my $mv_item = '';
-    $mv_frame -> Button(-text => 'Move selected item into:',
-			-command => sub {$cw->move_selected_to($mv_item);},
-		      	-anchor => 'e',
-		       )
-              -> pack(-side => 'left', @fxe1);
-    $mv_frame -> Entry (-textvariable => \$mv_item, -width => $entry_width)
-              -> pack  (-side => 'left') ;
+    my $mv_b = $button_frame 
+      -> Button(-text => 'Move',
+		-command => sub { $cw->move_selected_to($item); 
+				  $item = '' unless $keep; },
+		-anchor => 'e',
+	       )
+	-> pack(-side => 'left');
+    $balloon->attach($mv_b,
+		     -msg => "move selected item in entry");
 
     if ($hash->ordered) {
 	my $mv_up_down_frame = $right_frame->Frame->pack( @fxe1);
@@ -137,16 +161,18 @@ sub Populate {
 				 )-> pack( -side =>'left' , @fxe1);
     }
 
-    my $del_rm_frame =  $right_frame->Frame->pack( @fxe1);
+    my $del_rm_frame =  $right_frame->Frame->pack( @fxe1, qw/-side top -anchor n/);
 
     $del_rm_frame->Button(-text => 'Delete selected',
-			  -command => sub { $cw->delete_selection ;} ,
+			  -command => sub { $cw->delete_selection; 
+					    $item = '' unless $keep;} ,
 			 )-> pack( -side =>'left' , @fxe1);
 
-    $del_rm_frame -> Button ( -text => 'Remove all elements',
+    $del_rm_frame -> Button ( -text => 'Remove all',
 			      -command => sub { $hash->clear ; 
 						$tklist->delete(0,'end');
 						$cw->reload_tree;
+						$item = '';
 					    },
 			    ) -> pack(-side => 'left', @fxe1) ;
 
@@ -420,7 +446,7 @@ sub store {
 
 sub reload_tree {
     my $cw = shift ;
-    $cw->parent->parent->parent->parent->reload(1) ;
+    $cw->{store_cb}->() ;
 }
 
 
