@@ -1,6 +1,6 @@
 # $Author: ddumont $
-# $Date: 2009-03-31 13:41:21 +0200 (Tue, 31 Mar 2009) $
-# $Revision: 910 $
+# $Date: 2009-07-31 16:37:39 +0200 (Fri, 31 Jul 2009) $
+# $Revision: 1009 $
 
 #    Copyright (c) 2008-2009 Dominique Dumont.
 #
@@ -30,7 +30,7 @@ use Log::Log4perl;
 use base qw/Config::Model::Tk::LeafViewer/;
 use vars qw/$VERSION/ ;
 
-$VERSION = sprintf "1.%04d", q$Revision: 910 $ =~ /(\d+)/;
+$VERSION = sprintf "1.%04d", q$Revision: 1009 $ =~ /(\d+)/;
 
 Construct Tk::Widget 'ConfigModelLeafEditor';
 
@@ -53,6 +53,7 @@ sub Populate {
     my $leaf = $cw->{leaf} = delete $args->{-item} 
       || die "LeafEditor: no -item, got ",keys %$args;
     delete $args->{-path} ;
+    $cw->{store_cb} = delete $args->{-store_cb} || die __PACKAGE__,"no -store_cb" ;
 
     my $inst = $leaf->instance ;
     $inst->push_no_value_check('fetch') ;
@@ -235,7 +236,7 @@ sub store {
     }
     else {
 	# trigger redraw of Tk Tree
-	$cw->parent->parent->parent->parent->reload(1) ;
+	$cw->{store_cb}->($cw->{leaf}) ;
     }
 }
 
